/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import org.apache.commons.io.FilenameUtils;
import com.openexchange.file.storage.Document;
import com.openexchange.file.storage.File;
import com.openexchange.file.storage.File.Field;

/**
 * {@link NameBasedEncryptedFileRecognizer} recognizes a file as encrypted based on the file extension
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.2
 */
public class NameBasedEncryptedFileRecognizer implements EncryptedFileRecognizer {

    private static final String[] EXTENSIONS = new String[] { "pgp" };
    private static final String MARK_EXTENSION = ".pgp";
    private final boolean caseSensitive;

    /**
     * Initializes a new {@link NameBasedEncryptedFileRecognizer}.
     */
    public NameBasedEncryptedFileRecognizer() {
        this(false);
    }

    /**
     * Initializes a new {@link NameBasedEncryptedFileRecognizer}.
     *
     * @param caseSensitive True, if name based recognition should be case sensitive, false otherwise.
     */
    public NameBasedEncryptedFileRecognizer(boolean caseSensitive) {
        this.caseSensitive = caseSensitive;
    }

    /**
     * Internal method for comparing
     *
     * @param one
     * @param two
     * @return True, if the two strings are equal, false otherwise.
     */
    private boolean areEquals(String one, String two) {
        if (one == null && two == null) {
            return true;
        }
        if (one != null && two != null) {
            if (this.caseSensitive) {
                return one.equals(two);
            } else {
                return one.toLowerCase().equals(two.toLowerCase());
            }
        }
        return false;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.file.storage.GuardFileRecognizer#isGuardFile(com.openexchange.file.storage.Document)
     */
    @Override
    public boolean isEncrypted(File file) {
        String documentExtension = FilenameUtils.getExtension(file.getFileName());
        for (String extension : EXTENSIONS) {
            if (areEquals(extension, documentExtension)) {
                return true;
            }
        }
        return false;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markEncrypted(com.openexchange.file.storage.File)
     */
    @Override
    public Collection<Field> markEncrypted(File file) {
        ArrayList<Field> ret = new ArrayList<Field>();
        if (file.getFileName() != null && !file.getFileName().endsWith(MARK_EXTENSION)) {
            file.setFileName(file.getFileName() + MARK_EXTENSION);
            ret.add(Field.FILENAME);
        }
        return ret;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markEncrypted(com.openexchange.file.storage.Document)
     */
    @Override
    public Collection<Field> markEncrypted(Document document) {
        if(document.getName() != null && !document.getName().endsWith(MARK_EXTENSION)) {
            document.setName(document.getName() + MARK_EXTENSION);
        }
        return markEncrypted(document.getFile());
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markDecrypted(com.openexchange.file.storage.File)
     */
    @Override
    public Collection<Field> markDecrypted(File file) {
        String oldFileName = file.getFileName();
        if (oldFileName != null && oldFileName.endsWith(MARK_EXTENSION)) {
            String newFileName = oldFileName.substring(0, oldFileName.length() - MARK_EXTENSION.length());
            file.setFileName(newFileName);
        }
        return Arrays.asList(new Field[] { Field.FILENAME });
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.EncryptedFileRecognizer#markDecrypted(com.openexchange.file.storage.Document)
     */
    @Override
    public Collection<Field> markDecrypted(Document document) {
        String oldFileName = document.getName();
        if (oldFileName != null && oldFileName.endsWith(MARK_EXTENSION)) {
            String newFileName = oldFileName.substring(0, oldFileName.length() - MARK_EXTENSION.length());
            document.setName(newFileName);
        }
        return markDecrypted(document.getFile());
    }
}
